package org.jtag.dialog;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.util.*;
import java.sql.*;

import org.jtag.*;
import org.jtag.table.*;
import org.jtag.mp3.*;


public class JTagDbDialog extends JDialog
    implements ActionListener {
	
    String mUser,
        mPass,
        mURL,
        mDriver;
	
    mainWindow mParent;
	
    JTagTable mTable;
	
    JRadioButton radioTable,
        radioNewTable;
	
    JButton okButton;
	
    JTextField tableField;
	
    JComboBox tableCombo;
	
    Connection con;
	
    Vector foundTables = new Vector();

    public JTagDbDialog(String title, boolean modal, 
        boolean resize, String user, String pass,
        String url, String driver,
        JTagTable table, mainWindow parent) {

        setTitle(title);
        setModal(modal);
        setResizable(resize);

        mUser = user;
        mPass = pass;
        mURL = url;
        mDriver = driver;
        mTable = table;
   	
        mParent = parent;
   	
        openDb();
   	
        getTables();
   	
        closeDb();
   	  
        doUI();
    }
	
    public void doUI() {
        JPanel contentPane = (JPanel) getContentPane();

        contentPane.setLayout(new BorderLayout());  
   	
        radioTable = new JRadioButton("Add to existing table:");
        radioTable.setSelected(true);
        radioNewTable = new JRadioButton("Create new table:");
        ButtonGroup bgTable = new ButtonGroup();

        bgTable.add(radioTable);
        bgTable.add(radioNewTable);

        tableCombo = new JComboBox(foundTables);
   	  
        JPanel namePanel = new JPanel(false);

        namePanel.setLayout(new GridLayout(3, 2));
   	  
        JPanel fieldPanel = new JPanel(false);

        fieldPanel.setLayout(new FlowLayout());
   	 
        tableField = new JTextField();
   
        okButton = new JButton("Ok");
        okButton.addActionListener(this);
        okButton.setActionCommand("Ok");

        fieldPanel.add(okButton);
   	
        namePanel.add(radioTable);
        namePanel.add(tableCombo);
        namePanel.add(radioNewTable);
        namePanel.add(tableField);
        namePanel.add(new JLabel(""));
        namePanel.add(new JLabel(""));
      
        contentPane.add(namePanel, BorderLayout.CENTER);
        contentPane.add(fieldPanel, BorderLayout.SOUTH);
	 	
        // Create border
        CompoundBorder border = new CompoundBorder(
                new EmptyBorder(10, 10, 10, 10),
                new TitledBorder(new EtchedBorder(),
                    "Select database table"));

        contentPane.setBorder(border);
	 	
        this.setSize(400, 200);

    }	
	
    public void openDb() {
        try {
            // ODBC/JDBC-Bridge-Treiber laden:
            Class.forName(mDriver);     
    
            con = DriverManager.getConnection(mURL, mUser, mPass);
        } catch (Exception e) {
            mParent.mTextArea.append(e.toString() + "\r\n");
        }

    }
	
    public void closeDb() {
        try {
            con.close();
        } catch (Exception e) {
            mParent.mTextArea.append(e.toString() + "\r\n");
        }
    }
	
    public void getTables() {
        try {
            // Get the metadata
            DatabaseMetaData dbmd = con.getMetaData();
            //System.out.println("Database Product: " + dbmd.getDatabaseProductName());
	
            // Get the tables in the database
            // Normally it should work with getTables(null, "%", "%", null),
            // but somehow due to an error in MS Access or in the ODBC 
            // bridge we have to use the statement below
            // Now we define which types of tables we want (only TABLE)
            String tableTypes[] = {"TABLE"};
            ResultSet rsTables = dbmd.getTables(null, null, null, tableTypes);
            ResultSetMetaData rsmdTables = rsTables.getMetaData();
	  
            int cols = rsmdTables.getColumnCount();
		
            int tables = 0;
		
            while (rsTables.next()) {
                tables++;
                for (int i = 1; i <= cols; i++) {
                    if (rsmdTables.getColumnLabel(i).equals("TABLE_NAME")) {
                        String tableName = new String(rsTables.getString(i));
  	     	 	 
                        //System.out.println(tableName);
                        foundTables.addElement(tableName);
                    }
                }
            }	
	
            //System.out.println("Found " + tables +" tables in database");

        } catch (Exception e) {	
            mParent.mTextArea.append(e.toString() + "\r\n");
        }
    }
	
    public void createTable(String tableName) {
        String sqlStatement = new String("CREATE TABLE " + tableName + "( ");
        int size = 0;
		
        try {
            for (int i = 0; i < mParent.mOutputVector.size(); i++) { 
                String value = (String) mParent.mOutputVector.elementAt(i);
     		
                if (value.equals("Filename"))
                    size = 255;
                else
                    size = 30;
     		
                value = substString(value, '(', '_');
                value = substString(value, ')', '_');
                value = substString(value, ' ', '_');
                value = substString(value, ':', '_');
                //System.out.println("Value: " + value);    		
                sqlStatement = sqlStatement + 
                        value +
                        " CHAR(" + size + ")";
   	  	  
                if (i != mParent.mOutputVector.size() - 1)
                    sqlStatement = sqlStatement + ",";
            }
            sqlStatement = sqlStatement + ")";
			
            //System.out.println("SQL Statement: " + sqlStatement);
			
            Statement stmt = con.createStatement();

            stmt.execute(sqlStatement);
            stmt.close();
        } catch (Exception e) {
            mParent.mTextArea.append(e.toString() + "\r\n");
        }
		
    }
	
    public void insertTable(String tableName) {
        try {
            int numRows = mTable.getRowCount();
            int numCols = mTable.getColumnCount();
            TableModel model = mTable.getModel();
    
            mParent.resetProgressBar();
            mParent.setProgressBarMinMax(0, numRows);
    
            for (int row = 0; row < numRows; row++) {
                mParent.setProgressBarValue(row);

                String line = new String("INSERT INTO " + tableName +
                        " VALUES (");
                String column = new String();
    	
                for (int i = 0; i < mParent.mOutputVector.size(); i++) {

                    String item = (String) mParent.mOutputVector.elementAt(i);
   	  	
                    if (item.equals("Song title"))
                        column = model.getValueAt(row, 0).toString();
                    else if (item.equals("Artist"))
                        column = model.getValueAt(row, 1).toString();
                    else if (item.equals("Album"))
                        column = model.getValueAt(row, 2).toString();
                    else if (item.equals("Year"))
                        column = model.getValueAt(row, 3).toString();
                    else if (item.equals("Comment"))
                        column = model.getValueAt(row, 4).toString();
                    else if (item.equals("Genre")) {
                        column = model.getValueAt(row, 5).toString();
                        if (column.equals(" "))
                            column = "";
                    } else if (item.equals("Filename"))
                        column = model.getValueAt(row, 6).toString();
                    else if (item.equals("Directory"))
                        column = mTable.currentDirectory;
                    else {
                        //String fileName = new String(model.getValueAt(row,6).toString());
                        //Integer index = (Integer)mParent.fileHash.get(fileName);
                        //mp3Info inf = (mp3Info)mParent.fileVector.elementAt(index.intValue());
                        Integer index = new Integer((String) model.getValueAt(row, 7));
                        mp3Info inf = (mp3Info) mParent.fileVector.elementAt(index.intValue());

                        column = inf.getMp3HeaderValue(item);
                    }
   	  	
                    // Sorry SQL doesn't like '
                    column = substString(column, '\'', '');
   	    	
                    if (i != mParent.mOutputVector.size() - 1)
                        line = line + "'" + column + "'" + ",";
                    else
                        line = line + "'" + column + "'";
   	 
                }
                line = line + ")";
      	
                //System.out.println(line);
      	
                executeStatement(line);
    	
            }
   	
            mParent.resetProgressBar();
   	
        } catch (Exception e) {
            mParent.mTextArea.append(e.toString() + "\r\n");
        }
    }
	
    public void executeStatement(String statement) {
        try {
            Statement stmt = con.createStatement();

            stmt.execute(statement);
            stmt.close();
        } catch (Exception e) {
            mParent.mTextArea.append(e.toString() + "\r\n");
        }
    }
	
    public String substString(String string, char substChar, char substWith) {
        char[] charString = string.toCharArray();
		
        for (int i = 0; i < string.length(); i++) {
            if (charString[i] == substChar)
                charString[i] = substWith;
        }
        return (new String(charString));
    }
   	 
    public void actionPerformed(ActionEvent e) {
        String tableName = new String();
		
        if (e.getActionCommand().equals("Ok")) {
            this.setVisible(false);
  	
            openDb();
  		
            if (radioTable.isSelected()) {
                tableName = (String) tableCombo.getSelectedItem();
            } else {
                tableName = tableField.getText();
                tableName = substString(tableName, '(', '_');
                tableName = substString(tableName, ')', '_');
                tableName = substString(tableName, '-', '_');
                tableName = substString(tableName, ' ', '_');
                //System.out.println("Tablename: " + tableName);
                createTable(tableName);
            }		
  		
            // Substitute the spaces in table name
            tableName = substString(tableName, '(', '_');
            tableName = substString(tableName, ')', '_');
            tableName = substString(tableName, '-', '_');
            tableName = substString(tableName, ' ', '_');
  		
            //System.out.println(tableName);
            insertTable(tableName);	
            this.dispose();
            closeDb();
            return;
        }
    }
}
