package org.jtag.tools.concat;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;


/**
 * Concatenates MP3 files, including subfolders (if specified).
 * 
 * @author	Jochen Luell
 * 	
 */
public class ConcatMp3 extends JFrame implements ActionListener, Runnable
{
   String outputFileName = "concat.mp3";

   JProgressBar progressBar = new JProgressBar();
   JTextField directoryField = new JTextField("", 15);
   JButton startButton;
   JLabel progressLabel = new JLabel("    ");
   JLabel statusLabel = new JLabel("                               ");
   JTextArea textArea;
   JCheckBox recurseCb;
   boolean isApplication = false;

   /**
    * Main function of ConcatMP3.
    * 
    * @param	args
    * 	
    */
   public static void main(String args[])
   {
      ConcatMp3 frame = new ConcatMp3();
       frame.setSize(600, 400);
      frame.isApplication = true;
      frame.show();
/*
      try
      {
         concat.walkDir(new File(args[0]), true);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
*/
   }

  /**
   * ConcatMP3's constructor. Calls doUI().
   * 
   * 	
   */
  public ConcatMp3()
  {
     doUI();
  }

  /**
   * ConcatMP3's constructor. <br>Takes frame title as argument.
   * Calls doUI().
   * 
   * @param	title
   * 	
   */
  public ConcatMp3(String title)
  {
     super(title);
     doUI();
  }

   /**
    * Paints GUI elements.
    * 
    * 	
    */
   public void doUI()
   {
      getContentPane().setLayout(new BorderLayout());

      progressBar.setMinimum(0);


      //listPanel.add(new Label("Choose file:"));

      JPanel filePanel = new JPanel();
      filePanel.add(new JLabel("Directory:"));
      filePanel.add(directoryField);


      JButton browseButton = new JButton("Browse");
      browseButton.addActionListener(this);
      browseButton.setActionCommand("browse");

      recurseCb = new JCheckBox("Include subdirectories", true);

      //Browse button is disabled, because it allows only to select files, not directories
      filePanel.add(browseButton);

       filePanel.add(recurseCb);

      //Panel buttonPanel = new Panel();
      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));


      startButton = new JButton("Start");
      buttonPanel.add(startButton);
      startButton.addActionListener(this);
      startButton.setActionCommand("start");

      JPanel statusPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
      statusPanel.add(progressBar);
      statusPanel.add(progressLabel);
      statusPanel.add(statusLabel);

      JPanel northPanel = new JPanel(new GridLayout(2, 1));
      northPanel.add(statusPanel);
      northPanel.add(buttonPanel);

      textArea = new JTextArea();
      textArea.setEditable(false);
      JScrollPane scroller = new JScrollPane(textArea);

      getContentPane().add(filePanel, BorderLayout.NORTH);
      getContentPane().add(scroller, BorderLayout.CENTER);
      getContentPane().add(northPanel, BorderLayout.SOUTH);

      // Windows listener
      this.addWindowListener(new WindowAdapter()
                             {
                                public void windowClosing(WindowEvent e)
                                {
                                      if(isApplication)
                                         System.exit(0);
                                }
                             }
                            );
   }

   // Read directories
   private void walkDir(File dir,
           boolean recursive) throws FileNotFoundException, IOException
   {
      String list[] = dir.list();
      Vector fileVector = new Vector();

      if(list == null)
     {
        textArea.append("No files found !\n");
         return;
     }

     textArea.append("Entering directory: " + dir.toString() + "\n");

      for (int i = 0; i < list.length; i++)
      {
         File curFile = new File(dir.toString() + File.separator + list[i]);

         if (curFile.isDirectory())
         {
            // Walk recurively if specified
            if (recursive)
            {
               walkDir(curFile, recursive);
            }
         }
         else if (curFile.isFile())
         {
            // If it's a file, check if it has an mp3 extension
            if (curFile.toString().endsWith("mp3") ||
                    curFile.toString().endsWith("MP3"))
            {
               fileVector.add(list[i]);
            }
         }
         else
         {
            // Shouldn't happen, but just in case...
            System.out.println("Unknown file ... " + list[i]);
         }
      }

      // Sort the file vector. We do not have to provide a Comperator, because String implements Comparable
      Collections.sort(fileVector);

      // Delete restult file, if it exists
      File resFile = new File(dir + File.separator + outputFileName);
      if (resFile.exists())
         resFile.delete();

      for (Enumeration e = fileVector.elements(); e.hasMoreElements();)
      {
         String file = (String) e.nextElement();
         append(dir.toString(), file);
      }
   }

   /**
    * Appends MP3 files.
    * Here the real work is done. Traverses throught a directory
    * (including subdirectories if specified) and concatentes
    * the MP3 files.
    * 
    * @param	dir
    * @param	file
    * @exception	IOException
    * @exception	FileNotFoundException
    * 	
    */
   public void append(String dir,
           String file) throws FileNotFoundException, IOException
   {
      boolean newFile = false;
      boolean hasIdTag = false;
      byte[] tagBytes = new byte[3];
      int blockSize = 4096;
      byte[] buffer = new byte[blockSize];



      final int tagLength = 128;

      if (file.equals(outputFileName))
         return;

      textArea.append("\tProcessing file: " + file + "\n");
      progressBar.setValue(0);
      progressLabel.setText(" " + file);
     

      File output = new File(dir + File.separator + outputFileName);
      File input = new File(dir + File.separator + file);

      // If output file does not exist, create it
      if (!output.exists())
         newFile = true;

      RandomAccessFile inputStream = new RandomAccessFile(input, "r");
      RandomAccessFile outputStream = new RandomAccessFile(output, "rw");



      // To append set fp to end of output file
      outputStream.seek(output.length());

      // Get filesize
      long fileSize = input.length();

      // Go to TAG location
      if (fileSize > tagLength)
      {
         inputStream.seek(fileSize - tagLength);

         // Check if tag "TAG" is available
         int read = inputStream.read(tagBytes);
         String tagString = new String(tagBytes);

         if (tagString.compareTo("TAG") == 0)
         {
            hasIdTag = true;
            fileSize -= tagLength;
         }
      }
      else
         hasIdTag = false;

      // Reset FilePointer
      //if (newFile)
      inputStream.seek(0);
      //else if (fileSize >= 4)
      //{
      //  inputStream.seek(4); // Skip 32 bit MP3 header
      //  fileSize -= 4;
      //}

      int runs = (int) fileSize / blockSize;

     progressBar.setMaximum(runs + 1);

      for (int i = 0; i < runs; i++)
      {
         inputStream.read(buffer);
         outputStream.write(buffer);
         progressBar.setValue(i + 1);
      }

      //Now handle the rest of the file
      int rest = (int)(fileSize - runs * blockSize);

      if (rest > 0)
      {
         inputStream.read(buffer, 0, rest);
         outputStream.write(buffer, 0, rest);
      }
    
      progressBar.setValue(runs + 1);

      inputStream.close();
      outputStream.close();

   }

   /**
    * Displays directory chooser dialog.
    * 
    * 	
    */
   public String showDirectoryChooser()
   {
      //JFrame f = new JFrame();
      //FileDialog fd = new FileDialog(f, "Choose directory");
      //fd.show();

      //return fd.getDirectory();

      JFileChooser dirChooser = new JFileChooser();
      dirChooser.setDialogTitle("Choose target directory");
      dirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

      int returnValue = dirChooser.showOpenDialog(this);

      if (returnValue == JFileChooser.CANCEL_OPTION)
         return "";

      File inputDir = dirChooser.getSelectedFile();

      return inputDir.toString();
   }

   /**
    * Thread run method.
    * 
    * 	
    */
   public void run()
   {
      try {
        startButton.setEnabled(false);
         textArea.setText("Result file name: " + outputFileName + "\n\n");
         walkDir(new File(directoryField.getText()), recurseCb.isSelected());
         textArea.append("\nFinished !\n");
         resetProgressBar();
         startButton.setEnabled(true);
      } catch(Exception e) { e.printStackTrace(); }
   }

 private void resetProgressBar()
   {
       progressLabel.setText("      ");
       progressBar.setValue(0);
   }

   /**
    * Action handler.
    * 
    * @param	e
    * 	
    */
   public void actionPerformed(ActionEvent e)
   {
      String command = e.getActionCommand();

      if (command.equals("browse"))
         directoryField.setText(showDirectoryChooser());

      if (command.equals("start"))
      {
         Thread thread = new Thread(this);
         thread.start();
      }

   }

}


